#include "oled.h"

#define OLED_RD PTDD_PTDD0
#define OLED_WR PTCD_PTCD1
#define OLED_CS PTCD_PTCD6
#define OLED_RES PTCD_PTCD5
#define OLED_A0 PTCD_PTCD4
#define OLED_DATA PTBD
#define OLED_DATA_DIR PTBDD

unsigned char oledSendData(unsigned char data) {
  OLED_A0 = 1;
  OLED_CS = 0;
  OLED_DATA = data;
  OLED_WR = 0;
  OLED_WR = 1;
  OLED_CS = 1;
  
  return 1;
}
  
unsigned char oledSendCommand(unsigned char command) {
  OLED_A0 = 0;
  OLED_CS = 0;
  OLED_DATA = command;
  OLED_WR = 0;
  OLED_WR = 1;
  OLED_CS = 1;
    
  return 1;
}
  
unsigned char oledSendDoubleCommand(int command) {
  OLED_A0 = 0;
  OLED_CS = 0;
  OLED_DATA = (char) (command >> 8);
  OLED_WR = 0;
  OLED_WR = 1;
  OLED_DATA = (char) (command & 0xFF);
  OLED_WR = 0;
  OLED_WR = 1;
  OLED_CS = 1;
   
  return 1;
}
  
 
unsigned char oledTurnOff(void) {
  oledSendCommand(0b10101110);
  return 1;
}
  
unsigned char oledTurnOn(void) {
  oledSendCommand(0b10101111);
  return 1;
}


unsigned char oledSetStartLine(unsigned char line) {
  oledSendCommand(0b01000000 | (line & 0b111111));
  return 1;
}
 
unsigned char oledSetSide(unsigned char side) {
  oledSendCommand(0b10100000 | (side != 0));
  return 1;
}
  
  
unsigned char oledSetOffset(unsigned char offset) {
  oledSendDoubleCommand(0b1101001100000000 | offset);
  return 1;  
}

unsigned char oledSetColumn(unsigned char column) {
  oledSendDoubleCommand(((column & 0b1111) << 8) | (0b00010000 | (column >> 4)));
  return 1;  
}
  
unsigned char oledSetPage(unsigned char page) {
  oledSendCommand(0b10110000 | (page & 0b1111));
  return 1;  
}

unsigned char oledSetContrast(unsigned char contrast) {
  oledSendDoubleCommand(0b1000000100000000 | contrast);
  return 1;
}


unsigned char oledIsBusy(void) {
  unsigned char status;
  
  OLED_DATA_DIR = 0x00;
  OLED_A0 = 0;
  OLED_CS = 0;
  OLED_RD = 0;
  status = OLED_DATA;
  OLED_RD = 1;
  OLED_CS = 1;
  OLED_DATA_DIR = 0xFF;
    
  if(status & 0b10000000) {
    return 1;
  } else {
    return 0;
  }
}

unsigned char oledInit(void) {
  PTCDD_PTCDD4 = 1;
  PTCDD_PTCDD5 = 1;
  PTCDD_PTCDD6 = 1;
    
  PTCDD_PTCDD1 = 1;
  PTDDD_PTDDD0 = 1;
  
  OLED_DATA_DIR = 0xFF;
  OLED_DATA = 0x00;
    
  OLED_RD = 1;
  OLED_WR = 1;
  OLED_CS = 1;
  OLED_A0 = 1;
  OLED_RES = 0;
  wait(1); // wait routine
  OLED_RES = 1;
}